@@ -1,13 +1,14 @@
 from procrastinate import metadata as _metadata_module
-from procrastinate.aiopg_connector import PostgresConnector, PostgresJobStore
+from procrastinate.aiopg_connector import AiopgConnector
 from procrastinate.app import App
+from procrastinate.job_context import JobContext
 from procrastinate.retry import BaseRetryStrategy, RetryStrategy
 
 __all__ = [
     "App",
+    "JobContext",
     "BaseRetryStrategy",
-    "PostgresConnector",
-    "PostgresJobStore",
+    "AiopgConnector",
     "RetryStrategy",
 ]
 
