@@ -1,11 +1,10 @@
 import functools
 import logging
-import warnings
 from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Optional, Set
 
-from procrastinate import builtin_tasks
+from procrastinate import admin
 from procrastinate import connector as connector_module
-from procrastinate import exceptions, healthchecks, jobs
+from procrastinate import healthchecks, jobs
 from procrastinate import retry as retry_module
 from procrastinate import schema, store, utils
 
@@ -20,33 +19,52 @@ class App:
     """
     The App is the main entry point for procrastinate integration.
 
-    Instantiate a single :py:class:`App` in your code
-    and use it to decorate your tasks with :py:func:`App.task`.
+    Instantiate a single `App` in your code
+    and use it to decorate your tasks with `App.task`.
 
-    You can run a worker with :py:func:`App.run_worker`.
+    You can run a worker with `App.run_worker`.
+
+    Attributes
+    ----------
+    tasks : ``Dict[str, tasks.Task]``
+        The mapping of all tasks known by the app. Only procrastinate is expected to
+        make changes to this mapping.
+    builtin_tasks : ``Dict[str, tasks.Task]``
+        The mapping of builtin tasks. Use it to programmatically access builtin tasks,
+        to defer them.
+    admin : ``admin.Admin``
+        The administration interface linked to the application.
     """
 
     @classmethod
-    def from_path(cls, dotted_path: str):
+    def from_path(cls, dotted_path: str) -> "App":
+        """
+        Create an :py:class:`App` object by dynamically loading the
+        object at the given path.
+
+        Parameters
+        ----------
+        dotted_path :
+            Dotted path to the object to load (e.g.
+            ``mymodule.submodule.procrastinate_app``)
+        """
         return utils.load_from_path(dotted_path, cls)
 
     def __init__(
         self,
         *,
-        connector: Optional[connector_module.BaseConnector] = None,
+        connector: connector_module.BaseConnector,
         import_paths: Optional[Iterable[str]] = None,
-        # Just for backwards compatibility
-        job_store: Optional[connector_module.BaseConnector] = None,
+        worker_defaults: Optional[Dict] = None,
     ):
         """
         Parameters
         ----------
-        connector:
-            Instance of a subclass of :py:class:`BaseConnector`, typically
-            :py:class:`PostgresConnector`. It will be responsible for all
-            communications with the database.
-            Mandatory if job_store is not passed.
-        import_paths:
+        connector :
+            Instance of a subclass of :py:class:`procrastinate.connector.BaseConnector`,
+            typically `AiopgConnector`. It will be responsible for all communications
+            with the database. Mandatory.
+        import_paths :
             List of python dotted paths of modules to import, to make sure
             that the workers know about all possible tasks.
             If you fail to add a path here and a worker encounters
@@ -54,29 +72,19 @@ class App:
             fly and run, but you will get a warning.
             You don't need to specify paths that you know have already
             been imported, though it doesn't hurt.
-            A :py:func:`App.task` that has a custom "name" parameter, that is not
+            A `App.task` that has a custom "name" parameter, that is not
             imported and whose module path is not in this list will
             fail to run.
-        job_store:
-            **Deprecated**: Old name of ``connector``.
+        worker_defaults :
+            All the values passed here will override the default values sent when
+            launching a worker. See `App.run_worker` for details.
         """
-        # Compatibility
-        if job_store:
-            message = (
-                "Use App(connector=procrastinate.PostgresConnector(...)) "
-                "instead of App(job_store=procrastinate.PostgresJobStore())"
-            )
-            logger.warn(f"Deprecation Warning: {message}")
-            warnings.warn(DeprecationWarning(message))
-            connector = job_store
-        if not connector:
-            raise TypeError("App() missing 1 required argument: 'connector'")
-
         self.connector = connector
         self.tasks: Dict[str, "tasks.Task"] = {}
         self.builtin_tasks: Dict[str, "tasks.Task"] = {}
         self.queues: Set[str] = set()
         self.import_paths = import_paths or []
+        self.worker_defaults = worker_defaults or {}
 
         self.job_store = store.JobStore(connector=self.connector)
 
@@ -89,6 +97,7 @@ class App:
         queue: str = jobs.DEFAULT_QUEUE,
         name: Optional[str] = None,
         retry: retry_module.RetryValue = False,
+        pass_context: bool = False,
     ) -> Any:
         """
         Declare a function as a task. This method is meant to be used as a decorator::
@@ -119,15 +128,17 @@ class App:
             Name of the task, by default the full dotted path to the decorated function.
             if the function is nested or dynamically defined, it is important to give
             it a unique name, and to make sure the module that defines this function
-            is listed in the ``import_paths`` of the :py:class:`procrastinate.App`.
+            is listed in the ``import_paths`` of the `procrastinate.App`.
         retry :
             Details how to auto-retry the task if it fails. Can be:
 
             - A ``boolean``: will either not retry or retry indefinitely
             - An ``int``: the number of retries before it gives up
-            - A :py:class:`procrastinate.RetryStrategy` instance for complex cases
+            - A `procrastinate.RetryStrategy` instance for complex cases
 
             Default is no retry.
+        pass_context :
+            Passes the task execution context in the task as first
         """
         # Because of https://github.com/python/mypy/issues/3157, this function
         # is quite impossible to type consistently, so, we're just using "Any"
@@ -135,7 +146,14 @@ class App:
         def _wrap(func: Callable[..., "tasks.Task"]):
             from procrastinate import tasks
 
-            task = tasks.Task(func, app=self, queue=queue, name=name, retry=retry)
+            task = tasks.Task(
+                func,
+                app=self,
+                queue=queue,
+                name=name,
+                retry=retry,
+                pass_context=pass_context,
+            )
             self._register(task)
 
             return functools.update_wrapper(task, func)
@@ -156,6 +174,8 @@ class App:
             self.queues.add(queue)
 
     def _register_builtin_tasks(self) -> None:
+        from procrastinate import builtin_tasks
+
         builtin_tasks.register_builtin_tasks(self)
 
     def configure_task(self, name: str, **kwargs: Any) -> jobs.JobDeferrer:
@@ -169,7 +189,7 @@ class App:
             to the task (``my.module.my_task``)
 
         **kwargs: Any
-            Parameters from :py:func:`Task.configure`
+            Parameters from `Task.configure`
 
         Returns
         -------
@@ -180,12 +200,12 @@ class App:
 
         return tasks.configure_task(name=name, job_store=self.job_store, **kwargs)
 
-    def _worker(
-        self, queues: Optional[Iterable[str]] = None, name: Optional[str] = None
-    ) -> "worker.Worker":
+    def _worker(self, **kwargs) -> "worker.Worker":
         from procrastinate import worker
 
-        return worker.Worker(app=self, queues=queues, name=name)
+        final_kwargs = {**self.worker_defaults, **kwargs}
+
+        return worker.Worker(app=self, **final_kwargs)
 
     @functools.lru_cache(maxsize=1)
     def perform_import_paths(self):
@@ -199,36 +219,41 @@ class App:
             extra={"action": "imported_tasks", "tasks": list(self.tasks)},
         )
 
-    async def run_worker_async(
-        self,
-        queues: Optional[Iterable[str]] = None,
-        name: Optional[str] = None,
-        only_once: bool = False,
-    ) -> None:
+    async def run_worker_async(self, **kwargs) -> None:
         """
-        Run a worker. This worker will run in the foreground
-        and the function will not return until the worker stops
-        (most probably when it receives a stop signal) (except if
-        `only_once` is True).
+        Run a worker. This worker will run in the foreground and execute the jobs in the
+        provided queues. If wait is True, the function will not
+        return until the worker stops (most probably when it receives a stop signal).
+        The default values of all parameters presented here can be overriden at the
+        `App` level.
 
         Parameters
         ----------
-        queues:
-            List of queues to listen to, or None to listen to
-            every queue.
-        only_once:
-            If True, the worker will run but just for the currently
-            defined tasks. This function will return when the
-            listened queues are empty.
+        queues : ``Optional[Iterable[str]]``
+            List of queues to listen to, or None to listen to every queue (defaults to
+            ``None``).
+        wait : ``bool``
+            If False, the worker will terminate as soon as it has caught up with the
+            queues. If True, the worker will work until it is stopped by a signal
+            (ctrl+c, SIGINT, SIGTERM) (defaults to ``True``).
+        concurrency : ``int``
+            Indicates how many asynchronous jobs the worker can run in parallel.
+            Do not use concurrency if you have synchronous blocking tasks.
+            See `howto/concurrency` (defaults to ``1``).
+        name : ``Optional[str]``
+            Name of the worker. Will be passed in the `JobContext` and used in the
+            logs (defaults to ``None`` which will result in the worker named
+            ``worker``).
+        timeout : ``float``
+            Indicates the maximum duration (in seconds) procrastinate
+            workers wait between each database job poll.
+            Raising this parameter can lower the rate of workers making queries to the
+            database for requesting jobs.
+            (defaults to 5.0)
         """
-        worker = self._worker(queues=queues, name=name)
-        if only_once:
-            try:
-                await worker.process_jobs_once()
-            except (exceptions.NoMoreJobs, exceptions.StopRequested):
-                pass
-        else:
-            await worker.run()
+        self.perform_import_paths()
+        worker = self._worker(**kwargs)
+        await worker.run()
 
     @property
     def schema_manager(self) -> schema.SchemaManager:
@@ -238,5 +263,6 @@ class App:
     def health_check_runner(self) -> healthchecks.HealthCheckRunner:
         return healthchecks.HealthCheckRunner(connector=self.connector)
 
-    async def close_connection_async(self):
-        await self.connector.close_connection()
+    @property
+    def admin(self) -> admin.Admin:
+        return admin.Admin(connector=self.connector)
