@@ -1,15 +1,15 @@
+import asyncio
 import contextlib
-import datetime
 import json
 import logging
 import os
-from typing import Any, Callable, Dict, Iterable, Optional
+from typing import Any, Callable, Dict, Optional
 
 import click
 import pendulum
 
 import procrastinate
-from procrastinate import connector, exceptions, jobs, types
+from procrastinate import connector, exceptions, jobs, shell, types, utils, worker
 
 logger = logging.getLogger(__name__)
 
@@ -50,7 +50,9 @@ def handle_errors():
     try:
         yield
     except exceptions.ProcrastinateException as exc:
-        raise click.ClickException(str(exc))
+        logger.debug("Exception details:", exc_info=exc)
+        messages = [str(e) for e in utils.causes(exc)]
+        raise click.ClickException("\n".join(e for e in messages if e))
     except NotImplementedError:
         raise click.UsageError(
             "Missing app. This most probably happened because procrastinate needs an "
@@ -58,14 +60,6 @@ def handle_errors():
         )
 
 
-def print_version(ctx, __, value):
-    if not value or ctx.resilient_parsing:
-        return
-    click.echo(f"{PROGRAM_NAME} {procrastinate.__version__}")
-    click.echo(f"License: {procrastinate.__license__}")
-    ctx.exit()
-
-
 @click.group(context_settings=CONTEXT_SETTINGS)
 @click.pass_context
 @click.option("--app", "-a", help="Dotted path to the Procrastinate app")
@@ -77,17 +71,12 @@ def print_version(ctx, __, value):
     count=True,
     help="Use multiple times to increase verbosity",
 )
-@click.option(
-    "-V",
-    "--version",
-    is_flag=True,
-    callback=print_version,
-    expose_value=False,
-    is_eager=True,
+@click.version_option(
+    procrastinate.__version__, "-V", "--version", prog_name=PROGRAM_NAME
 )
-@handle_errors()
 def cli(ctx: click.Context, app: str, **kwargs) -> None:
     """
+@handle_errors()
     Interact with a Procrastinate app. See subcommands for details.
 
     All arguments can be passed by environment variables: PROCRASTINATE_UPPERCASE_NAME
@@ -95,44 +84,95 @@ def cli(ctx: click.Context, app: str, **kwargs) -> None:
     PROCRASTINATE_DEFER_UNKNOWN, ...).
     """
     if app:
-        ctx.obj = procrastinate.App.from_path(dotted_path=app)
+        app_obj = procrastinate.App.from_path(dotted_path=app)
     else:
         # If we don't provide an app, initialize a default one that will fail if it
         # needs its job store.
-        ctx.obj = procrastinate.App(connector=connector.BaseConnector())
+        app_obj = procrastinate.App(connector=connector.BaseConnector())
+    ctx.obj = app_obj
+
+    worker_defaults = app_obj.worker_defaults.copy()
+    worker_defaults["queues"] = ",".join(worker_defaults.get("queues") or [])
+    ctx.default_map = {"worker": worker_defaults}
 
 
 @cli.resultcallback()
 @click.pass_obj
 def close_connection(procrastinate_app: procrastinate.App, *args, **kwargs):
     # There's an internal click param named app, we can't name our variable "app" too.
-    procrastinate_app.close_connection()  # type: ignore
+    procrastinate_app.connector.close()
+
+    asyncio.get_event_loop().close()
 
 
-@cli.command()
+@cli.command("worker")
 @click.pass_obj
+@click.option("-n", "--name", default=worker.WORKER_NAME, help="Name of the worker")
+@click.option(
+    "-q",
+    "--queues",
+    default="",
+    help="Comma-separated names of the queues to listen "
+    "to (empty string for all queues)",
+)
+@click.option(
+    "-c",
+    "--concurrency",
+    type=int,
+    default=worker.WORKER_CONCURRENCY,
+    help="Number of parallel asynchronous jobs to process at once",
+)
+@click.option(
+    "-t",
+    "--timeout",
+    type=float,
+    default=worker.WORKER_TIMEOUT,
+    help="How long to wait for database event push before polling",
+)
+@click.option(
+    "-w",
+    "--wait/--one-shot",
+    default=True,
+    help="When all jobs have been processed, whether to "
+    "terminate or to wait for new jobs",
+)
 @handle_errors()
-@click.argument("queue", nargs=-1)
-@click.option("--name", help="Name of the worker")
-def worker(app: procrastinate.App, queue: Iterable[str], name: Optional[str]):
+def worker_(app: procrastinate.App, queues: str, **kwargs):
     """
     Launch a worker, listening on the given queues (or all queues).
+    Values default to App.worker_defaults and then App.run_worker() defaults values.
     """
-    queues = list(queue) or None
-    queue_names = ", ".join(queues) if queues else "all queues"
+    queue_list = [q.strip() for q in queues.split(",")] if queues else None
+    if queue_list is None:
+        queue_names = "all queues"
+    else:
+        queue_names = ", ".join(queue_list)
     click.echo(f"Launching a worker on {queue_names}")
-    app.run_worker(queues=queues, name=name)  # type: ignore
+    app.run_worker(queues=queue_list, **kwargs)  # type: ignore
 
 
 @cli.command()
 @click.pass_obj
-@handle_errors()
 @click.argument("task")
 @click.argument("json_args", required=False)
+@click.option(
+    "--queue", help="The queue for deferring. Defaults to the task's default queue"
+)
 @click.option(
     "--lock", help="A lock string. Jobs sharing the same lock will not run concurrently"
 )
-@click.option("--queue", help="The queue for deferring. Defaults to the task ")
+@click.option(
+    "--queueing-lock",
+    help="A string value. The defer operation will fail if there already is a job "
+    "waiting in the queue with the same queueing lock",
+)
+@click.option(
+    "-i",
+    "--ignore-already-enqueued/--no-ignore-already-enqueued",
+    default=False,
+    help="Exit with code 0 even if the queueing lock is already taken, while still "
+    "displaying an error (default false)",
+)
 @click.option("--at", help="ISO-8601 localized datetime after which to launch the job")
 @click.option(
     "--in", "in_", type=int, help="Number of seconds after which to launch the job"
@@ -141,11 +181,14 @@ def worker(app: procrastinate.App, queue: Iterable[str], name: Optional[str]):
     "--unknown/--no-unknown",
     help="Whether unknown tasks can be deferred (default false)",
 )
+@handle_errors()
 def defer(
     app: procrastinate.App,
     task: str,
     json_args: str,
     lock: Optional[str],
+    queueing_lock: Optional[str],
+    ignore_already_enqueued: bool,
     queue: Optional[str],
     at: Optional[str],
     in_: Optional[int],
@@ -170,6 +213,7 @@ def defer(
     # Build kwargs. Remove all None kwargs to use their default values.
     configure_kwargs = {
         "lock": lock,
+        "queueing_lock": queueing_lock,
         "schedule_at": schedule_at,
         "schedule_in": schedule_in,
         "queue": queue,
@@ -186,7 +230,12 @@ def defer(
     click.echo(f"Launching a job: {task}({str_kwargs})")
 
     # And launching the job
-    job_deferrer.defer(**args)  # type: ignore
+    try:
+        job_deferrer.defer(**args)  # type: ignore
+    except exceptions.AlreadyEnqueued as exc:
+        if not ignore_already_enqueued:
+            raise
+        click.echo(f"{exc} (ignored)")
 
 
 def filter_none(dictionnary: Dict) -> Dict:
@@ -207,7 +256,9 @@ def load_json_args(json_args: str, json_loads: Callable) -> types.JSONDict:
     return args
 
 
-def get_schedule_at(at: Optional[str]) -> Optional[datetime.datetime]:
+# return type should not be Optional[str], but there's a typing bug in
+# Pendulum 2.1.0 (https://github.com/sdispater/pendulum/issues/450)
+def get_schedule_at(at: Optional[str]) -> Optional[str]:
     if at is None:
         return None
 
@@ -243,7 +294,6 @@ def configure_job(
 
 @cli.command()
 @click.pass_obj
-@handle_errors()
 @click.option(
     "--apply",
     "action",
@@ -260,6 +310,7 @@ def configure_job(
     flag_value="migrations-path",
     help="Output the path to the directory containing the migration scripts",
 )
+@handle_errors()
 def schema(app: procrastinate.App, action: str):
     """
     Apply SQL schema to the empty database. This won't work if the schema has already
@@ -295,6 +346,16 @@ def healthchecks(app: procrastinate.App):
         click.echo(f"{status.value}: {count}")
 
 
+@cli.command("shell")
+@click.pass_obj
+@handle_errors()
+def shell_(app: procrastinate.App):
+    """
+    Administration shell for procrastinate.
+    """
+    shell.ProcrastinateShell(app.admin).cmdloop()
+
+
 def main():
     # https://click.palletsprojects.com/en/7.x/python3/
     os.environ.setdefault("LC_ALL", "C.UTF-8")
