@@ -1,13 +1,17 @@
-from typing import Any, Callable, Dict, List, Optional
+import asyncio
+from typing import Any, Callable, Dict, Iterable, List, Optional
 
-SOCKET_TIMEOUT = 5.0  # seconds
+QUEUEING_LOCK_CONSTRAINT = "procrastinate_jobs_queueing_lock_idx"
 
 
 class BaseConnector:
     json_dumps: Optional[Callable] = None
     json_loads: Optional[Callable] = None
 
-    async def close_connection(self) -> None:
+    async def close_async(self) -> None:
+        pass
+
+    def close(self) -> None:
         pass
 
     async def execute_query(self, query: str, **arguments: Any) -> None:
@@ -21,11 +25,7 @@ class BaseConnector:
     ) -> List[Dict[str, Any]]:
         raise NotImplementedError
 
-    def make_dynamic_query(self, query: str, **identifiers: str) -> str:
-        raise NotImplementedError
-
-    async def wait_for_activity(self) -> None:
-        raise NotImplementedError
-
-    def interrupt_wait(self):
+    async def listen_notify(
+        self, event: asyncio.Event, channels: Iterable[str]
+    ) -> None:
         raise NotImplementedError
