@@ -25,12 +25,16 @@ CREATE TABLE procrastinate_jobs (
     queue_name character varying(128) NOT NULL,
     task_name character varying(128) NOT NULL,
     lock text,
+    queueing_lock text,
     args jsonb DEFAULT '{}' NOT NULL,
     status procrastinate_job_status DEFAULT 'todo'::procrastinate_job_status NOT NULL,
     scheduled_at timestamp with time zone NULL,
     attempts integer DEFAULT 0 NOT NULL
 );
 
+-- this prevents from having several jobs with the same queueing lock in the "todo" state
+CREATE UNIQUE INDEX procrastinate_jobs_queueing_lock_idx ON procrastinate_jobs (queueing_lock) WHERE status = 'todo';
+
 CREATE TABLE procrastinate_events (
     id BIGSERIAL PRIMARY KEY,
     job_id integer NOT NULL REFERENCES procrastinate_jobs ON DELETE CASCADE,
@@ -69,7 +73,7 @@ BEGIN
 		FROM potential_job, lock_object
         WHERE lock_object.object IS NOT NULL
 		AND procrastinate_jobs.id = potential_job.id
-		RETURNING * INTO found_jobs;
+		RETURNING procrastinate_jobs.* INTO found_jobs;
 
 	RETURN found_jobs;
 END;
@@ -170,7 +174,7 @@ CREATE TRIGGER procrastinate_trigger_status_events_update
 
 CREATE TRIGGER procrastinate_trigger_status_events_insert
     AFTER INSERT ON procrastinate_jobs
-    FOR EACH ROW WHEN ((new.status = 'doing'::procrastinate_job_status))
+    FOR EACH ROW WHEN ((new.status = 'todo'::procrastinate_job_status))
     EXECUTE PROCEDURE procrastinate_trigger_status_events_procedure_insert();
 
 CREATE TRIGGER procrastinate_trigger_scheduled_events
