@@ -2,7 +2,8 @@ import asyncio
 import functools
 import importlib
 import logging
-from typing import Awaitable, Iterable, Type, TypeVar
+import types
+from typing import Any, Awaitable, Iterable, Optional, Type, TypeVar
 
 from procrastinate import exceptions
 
@@ -62,31 +63,81 @@ def add_sync_api(cls: Type) -> Type:
     return cls
 
 
+# https://github.com/sphinx-doc/sphinx/issues/7559
+SYNC_ADDENDUM = """
+
+        This method is the synchronous counterpart of `{}`.
+        Because of a slight issue in automatic doc generation, it
+        is shown here as "async", but this function is synchronous.
+"""
+
+ASYNC_ADDENDUM = """
+
+        This method is the asynchronous counterpart of `{}`.
+"""
+
+
 def wrap_one(cls: Type, attribute_name: str):
     suffix = "_async"
     if attribute_name.startswith("_") or not attribute_name.endswith(suffix):
         return
 
-    attribute = getattr(cls, attribute_name)
+    # Methods are descriptors so using getattr here will not give us the real method
+    cls_vars = vars(cls)
+    attribute = cls_vars[attribute_name]
+
+    # If method is a classmethod or staticmethod, its real function, that may be
+    # async, is stored in __func__.
+    wrapped = getattr(attribute, "__func__", attribute)
 
     # Keep only async def methods
-    if not asyncio.iscoroutinefunction(attribute):
+    if not asyncio.iscoroutinefunction(wrapped):
         return
 
+    attribute.__doc__ = attribute.__doc__ or ""
+
     # Create a wrapper that will call the method in a run_until_complete
-    @functools.wraps(attribute)
-    def wrapper(self, *args, **kwargs):
-        awaitable = attribute(self, *args, **kwargs)
+    @functools.wraps(wrapped)
+    def wrapper(*args, **kwargs):
+        awaitable = wrapped(*args, **kwargs)
         return sync_await(awaitable=awaitable)
 
+    sync_name = attribute_name[: -len(suffix)]
+    attribute.__doc__ += ASYNC_ADDENDUM.format(sync_name)
+
+    final_wrapper: Any
+    if isinstance(attribute, types.FunctionType):  # classic method
+        final_wrapper = wrapper
+    elif isinstance(attribute, classmethod):
+        final_wrapper = classmethod(wrapper)
+    elif isinstance(attribute, staticmethod):
+        final_wrapper = staticmethod(wrapper)
+    else:
+        raise ValueError(f"Invalid object of type {type(attribute)}")
+
     # Save this new method on the class
-    name = wrapper.__name__ = attribute_name[: -len(suffix)]
-    setattr(cls, name, wrapper)
+    name = wrapper.__name__ = sync_name
+    final_wrapper.__doc__ += SYNC_ADDENDUM.format(attribute_name)
+    setattr(cls, name, final_wrapper)
 
 
 def sync_await(awaitable: Awaitable[T]) -> T:
     """
     Given an awaitable, awaits it synchronously. Returns the result after it's done.
     """
+
     loop = asyncio.get_event_loop()
+    if loop.is_closed():
+        loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(loop)
     return loop.run_until_complete(awaitable)
+
+
+def causes(exc: Optional[BaseException]):
+    """
+    From a single exception with a chain of causes and contexts, make an iterable
+    going through every exception in the chain.
+    """
+    while exc:
+        yield exc
+        exc = exc.__cause__ or exc.__context__
